Package: cfid
Type: Package
Title: Identification of Counterfactual Queries in Causal Models
Version: 0.1.7
Authors@R: c(
    person(given = "Santtu",
           family = "Tikka",
           role = c("aut", "cre"),
           email = "santtuth@gmail.com",
           comment = c(ORCID = "0000-0003-4039-4342")))
Maintainer: Santtu Tikka <santtuth@gmail.com>
Description: Facilitates the identification of counterfactual queries in
 structural causal models via the ID* and IDC* algorithms
 by Shpitser, I. and Pearl, J. (2007, 2008) <arXiv:1206.5294>, 
 <https://jmlr.org/papers/v9/shpitser08a.html>. 
 Provides a simple interface for defining causal diagrams and counterfactual 
 conjunctions. Construction of parallel worlds graphs and counterfactual graphs 
 is carried out automatically based on the counterfactual query and the causal 
 diagram. See Tikka, S. (2023) <doi:10.32614/RJ-2023-053> for a tutorial of 
 the package.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/santikka/cfid
RoxygenNote: 7.2.3
Suggests: covr, igraph, mockery, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2023-11-27 15:16:28 UTC; santikka
Author: Santtu Tikka [aut, cre] (<https://orcid.org/0000-0003-4039-4342>)
Repository: CRAN
Date/Publication: 2023-11-27 16:00:04 UTC
Built: R 4.4.0; ; 2024-05-03 05:52:15 UTC; unix
